/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Peers management module.
 * 
 * 
 */

#ifndef _PEER_H
#define _PEER_H

/* Include the API for the peer manager */
#include <waaad/peer-api.h>

/* Include the definition of msg_t */
#include <waaad/message-api.h>

/* Include the definition of rt_dpl_t */
#include <waaad/routing-api.h>

/* Include the definition of uti_list_t */
#include "utils.h"

/* Functions called only in the daemon */

/*
 * FUNCTION:	peer_init
 *
 * PARAMETERS:
 *  -
 *
 * DESCRIPTION: 
 *  Initialize the peers management facility. 
 *
 * RETURN VALUE:
 *   0	: Application is now ready to use the peers management.
 *  !0  : An error occurred. 
 */
int peer_init ( void );

/*
 * FUNCTION:	peer_fini
 *
 * PARAMETERS:
 *  -
 *
 * DESCRIPTION: 
 *  Terminates the peers management facility. No peer_* function must be called after this one.
 *
 * RETURN VALUE:
 *   0 : peer facility closed properly.
 *  !0 : an error occurred (we may ignore it)
 */
int peer_fini ( void );


/*
 * FUNCTION:	peer_start
 *
 * PARAMETERS:
 *  -
 *
 * DESCRIPTION: 
 *  Allows the peer module to create the peer threads and to start connexions to other peers.
 *  Before this function is called, peer_add will only queue the information for later use.
 *  After this function has been called, following calls to peer_add will result in immediate
 *  creation of the peer thread and connexion.
 *
 * RETURN VALUE:
 *   0 : State changed, peer threads are starting.
 *  !0 : an error occurred.
 */
int peer_start ( void );


/*
 * FUNCTION:	peer_send
 *
 * PARAMETERS:
 *  peer  : The peer to which the message must be sent.
 *  msg   : The message that must be sent. 
 *
 * DESCRIPTION: 
 *   Queue a message for sending to a specific peer.
 *  If the peer is not in OPEN state, an error is returned.
 *  msg_update_length is called on the message, then it is queued and sent asynchronously.
 *  Note: if an error occurs while the message is being sent, the caller of this function is not notified.
 *
 * RETURN VALUE:
 *  0        : The message has been queued.
 *  EINVAL   : a parameter is invalid.
 *  ENOTCONN : the peer is not in OPEN state.
 */
int peer_send ( peer_t * peer, msg_t * msg );


/*
 * FUNCTION:	peer_get
 *
 * PARAMETERS:
 *  diamid	: The peer diameter identity
 *  hash	: Hash value for the peer
 *
 * DESCRIPTION:
 *  Find the peer structure corresponding to a given diamid / hash, if any.
 *
 * RETURN VALUE:
 *  Pointer to the peer structure or NULL if not found.
 */
peer_t * peer_struct_getptr(char * diamid, uint32_t hash);

/* List of peers that must NOT be in the rt_dpl_t list returned by next function */
/* Note that the list is ordered by hash and AVP value */
typedef struct {
	uti_list_t	chain;	/* linking information. The "o" field points to the AVP data location. */
	uint32_t	hash;	/* The hash value for this AVP data. */
} rt_reject_t;

/*
 * FUNCTION:	peer_list_open
 *
 * PARAMETERS:
 *  app    : the application that must be supported (or 0 for all peers)
 *  list   : location where the list must be stored.
 *  rejects: list of diameter ids of the peers that must not be returned.
 *
 * DESCRIPTION:
 *  Create a master list for routing module with all the peers in OPEN state (except rejected) 
 *  that support the application app (or relay), at the time of calling.
 *  Note that this function also frees the content of the rejects list as a side effect.
 *
 * RETURN VALUE:
 *   0 : List is created successfully.
 *  !0 : an error occurred.
 */
int peer_struct_list_open( application_id_t app, rt_dpl_t ** list, uti_list_t * rejects );


/* Helper function for debug */
void peer_dump(peer_t * peer, int debug_level, int what);

#endif /* ! _PEER_H */

